@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2009 Michael Kuhn
+ * Copyright (c) 2008-2012 Michael Kuhn
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -25,24 +25,33 @@
  * SUCH DAMAGE.
  */
 
-#include "maki.h"
+#define _XOPEN_SOURCE
 
-#include "dcc_send.h"
-#include "instance.h"
-#include "network.h"
-#include "server.h"
+#include "config.h"
 
-#include "ilib.h"
+#include <glib.h>
+#include <glib/gstdio.h>
+#include <gio/gio.h>
 
 #include <arpa/inet.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <netdb.h>
 #include <netinet/in.h>
+#include <string.h>
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <unistd.h>
 
+#include <ilib.h>
+
+#include "dcc_send.h"
+
+#include "dbus.h"
+#include "instance.h"
+#include "network.h"
+#include "server.h"
+
 enum
 {
 	s_new = (1 << 0),
@@ -536,9 +545,8 @@ makiDCCSend* maki_dcc_send_new_out (makiServer* serv, makiUser* user, const gcha
 {
 	guint i;
 	gchar* basename;
+	GInetAddress* inet_address;
 	struct stat stbuf;
-	makiNetworkAddress stun_addr;
-	socklen_t stun_addrlen = sizeof(stun_addr.ss);
 	makiInstance* inst = maki_instance_get_default();
 	makiNetwork* net = maki_instance_network(inst);
 	makiDCCSend* dcc;
@@ -600,14 +608,19 @@ makiDCCSend* maki_dcc_send_new_out (makiServer* serv, makiUser* user, const gcha
 	g_io_channel_set_close_on_unref(dcc->channel.connection, TRUE);
 	g_io_channel_set_encoding(dcc->channel.connection, NULL, NULL);
 
-	if (maki_network_remote_addr(net, &(stun_addr.sa), &stun_addrlen))
+	if ((inet_address = maki_network_external_address(net)) != NULL)
 	{
-		dcc->address = ntohl(stun_addr.sin.sin_addr.s_addr);
+		if (g_inet_address_get_family(inet_address) == G_SOCKET_FAMILY_IPV4)
+		{
+			dcc->address = ntohl(*((guint32 const*)g_inet_address_to_bytes(inet_address)));
+		}
+
+		g_object_unref(inet_address);
 	}
 	else
 	{
 		makiNetworkAddress addr;
-		socklen_t addrlen = sizeof(addr.ss);
+		socklen_t addrlen = sizeof(addr);
 
 		getsockname(g_io_channel_unix_get_fd(dcc->channel.connection), &(addr.sa), &addrlen);
 		dcc->address = ntohl(addr.sin.sin_addr.s_addr);
@@ -678,7 +691,7 @@ void maki_dcc_send_free (makiDCCSend* dcc)
 
 	maki_dcc_send_close(dcc);
 
-	maki_user_unref(dcc->user);
+	maki_server_remove_user(dcc->server, maki_user_nick(dcc->user));
 
 	g_free(dcc->path);
 	g_free(dcc);
@@ -986,12 +999,10 @@ gchar* maki_dcc_send_filename (makiDCCSend* dcc)
 void maki_dcc_send_emit (makiDCCSend* dcc)
 {
 	gchar* filename;
-	GTimeVal timeval;
 
-	g_get_current_time(&timeval);
 	filename = maki_dcc_send_filename(dcc);
 
-	maki_dbus_emit_dcc_send(timeval.tv_sec, dcc->id, maki_server_name(dcc->server), maki_user_from(dcc->user), filename, dcc->size, maki_dcc_send_progress(dcc), maki_dcc_send_speed(dcc), dcc->status);
+	maki_dbus_emit_dcc_send(dcc->id, maki_server_name(dcc->server), maki_user_from(dcc->user), filename, dcc->size, maki_dcc_send_progress(dcc), maki_dcc_send_speed(dcc), dcc->status);
 
 	g_free(filename);
 }
