@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2010 Michael Kuhn
+ * Copyright (c) 2008-2012 Michael Kuhn
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -25,21 +25,27 @@
  * SUCH DAMAGE.
  */
 
-#include "maki.h"
+#include "config.h"
 
-#include "dcc_send.h"
-#include "in.h"
-#include "instance.h"
-#include "misc.h"
-#include "out.h"
-#include "server.h"
-
-#include "ilib.h"
+#include <glib.h>
+#include <glib/gi18n.h>
 
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
 
+#include <ilib.h>
+
+#include "in.h"
+
+#include "dbus.h"
+#include "dcc_send.h"
+#include "instance.h"
+#include "maki.h"
+#include "misc.h"
+#include "out.h"
+#include "server.h"
+
 /* A convenience function to remove a colon before an argument.
  * It also checks for NULL. */
 static gchar* maki_remove_colon (gchar* string)
@@ -52,30 +58,31 @@ static gchar* maki_remove_colon (gchar* string)
 	return string;
 }
 
-static gboolean maki_mode_has_parameter (makiServer* serv, gchar sign, gchar mode)
+static gboolean
+maki_mode_has_parameter (makiServer* serv, gchar sign, gchar mode)
 {
-	gint type;
-	gchar* chanmode;
+	guint type;
+	const gchar* chanmodes;
 
-	if (serv->support.chanmodes == NULL)
-	{
-		return FALSE;
-	}
-
-	if (strchr(serv->support.prefix.modes, mode) != NULL)
+	if (strchr(maki_server_support(serv, MAKI_SERVER_SUPPORT_PREFIX_MODES), mode) != NULL)
 	{
 		return TRUE;
 	}
 
-	for (chanmode = serv->support.chanmodes, type = 0; *chanmode != '\0'; ++chanmode)
+	if ((chanmodes = maki_server_support(serv, MAKI_SERVER_SUPPORT_CHANMODES)) == NULL)
 	{
-		if (*chanmode == ',')
+		return FALSE;
+	}
+
+	for (type = 0; *chanmodes != '\0'; chanmodes++)
+	{
+		if (*chanmodes == ',')
 		{
-			++type;
+			type++;
 			continue;
 		}
 
-		if (*chanmode == mode)
+		if (*chanmodes == mode)
 		{
 			if (type == 0 || type == 1)
 			{
@@ -95,15 +102,17 @@ static gboolean maki_mode_has_parameter (makiServer* serv, gchar sign, gchar mod
 	return FALSE;
 }
 
-static gboolean maki_is_channel(makiServer* serv, const gchar* target)
+static gboolean
+maki_is_channel(makiServer* serv, const gchar* target)
 {
-	return (strchr(serv->support.chantypes, target[0]) != NULL);
+	return (strchr(maki_server_support(serv, MAKI_SERVER_SUPPORT_CHANTYPES), target[0]) != NULL);
 }
 
-static gint maki_prefix_position (makiServer* serv, gboolean is_prefix, gchar prefix)
+static gint
+maki_prefix_position (makiServer* serv, gboolean is_prefix, gchar prefix)
 {
 	guint pos = 0;
-	gchar* str = (is_prefix) ? serv->support.prefix.prefixes : serv->support.prefix.modes;
+	const gchar* str = (is_prefix) ? maki_server_support(serv, MAKI_SERVER_SUPPORT_PREFIX_PREFIXES) : maki_server_support(serv, MAKI_SERVER_SUPPORT_PREFIX_MODES);
 
 	while (str[pos] != '\0')
 	{
@@ -168,7 +177,7 @@ static void maki_commands (makiServer* serv)
 	g_strfreev(commands);
 }
 
-static void maki_in_dcc_send (makiServer* serv, glong timestamp, makiUser* user, gchar* remaining)
+static void maki_in_dcc_send (makiServer* serv, makiUser* user, gchar* remaining)
 {
 	gchar* file_name;
 	gsize file_name_len;
@@ -219,7 +228,7 @@ static void maki_in_dcc_send (makiServer* serv, glong timestamp, makiUser* user,
 	}
 }
 
-static void maki_in_dcc_resume_accept (makiServer* serv, glong timestamp, makiUser* user, gchar* remaining, gboolean is_incoming)
+static void maki_in_dcc_resume_accept (makiServer* serv, makiUser* user, gchar* remaining, gboolean is_incoming)
 {
 	gchar* file_name;
 	gsize file_name_len;
@@ -274,7 +283,7 @@ static void maki_in_dcc_resume_accept (makiServer* serv, glong timestamp, makiUs
 	}
 }
 
-static void maki_in_privmsg (makiServer* serv, glong timestamp, makiUser* user, gchar* remaining)
+static void maki_in_privmsg (makiServer* serv, makiUser* user, gchar* remaining)
 {
 	gchar** tmp;
 	gchar* target;
@@ -304,14 +313,14 @@ static void maki_in_privmsg (makiServer* serv, glong timestamp, makiUser* user,
 			{
 				if (maki_is_channel(serv, target))
 				{
-					maki_log(serv, target, "%s %s", maki_user_nick(user), message + 7);
+					maki_server_log(serv, target, "%s %s", maki_user_nick(user), message + 7);
 				}
 				else
 				{
-					maki_log(serv, maki_user_nick(user), "%s %s", maki_user_nick(user), message + 7);
+					maki_server_log(serv, maki_user_nick(user), "%s %s", maki_user_nick(user), message + 7);
 				}
 
-				maki_dbus_emit_action(timestamp, maki_server_name(serv), maki_user_from(user), target, message + 7);
+				maki_dbus_emit_action(maki_server_name(serv), maki_user_from(user), target, message + 7);
 			}
 			else
 			{
@@ -327,49 +336,49 @@ static void maki_in_privmsg (makiServer* serv, glong timestamp, makiUser* user,
 					}
 					else if (strncmp(message, "DCC SEND ", 9) == 0)
 					{
-						maki_in_dcc_send(serv, timestamp, user, message + 9);
+						maki_in_dcc_send(serv, user, message + 9);
 					}
 					else if (strncmp(message, "DCC RESUME ", 11) == 0)
 					{
-						maki_in_dcc_resume_accept(serv, timestamp, user, message + 11, FALSE);
+						maki_in_dcc_resume_accept(serv, user, message + 11, FALSE);
 					}
 					else if (strncmp(message, "DCC ACCEPT ", 11) == 0)
 					{
-						maki_in_dcc_resume_accept(serv, timestamp, user, message + 11, TRUE);
+						maki_in_dcc_resume_accept(serv, user, message + 11, TRUE);
 					}
 				}
 
 				if (maki_is_channel(serv, target))
 				{
-					maki_log(serv, target, "=%s= %s", maki_user_nick(user), message);
+					maki_server_log(serv, target, "=%s= %s", maki_user_nick(user), message);
 				}
 				else
 				{
-					maki_log(serv, maki_user_nick(user), "=%s= %s", maki_user_nick(user), message);
+					maki_server_log(serv, maki_user_nick(user), "=%s= %s", maki_user_nick(user), message);
 				}
 
-				maki_dbus_emit_ctcp(timestamp, maki_server_name(serv), maki_user_from(user), target, message);
+				maki_dbus_emit_ctcp(maki_server_name(serv), maki_user_from(user), target, message);
 			}
 		}
 		else
 		{
 			if (maki_is_channel(serv, target))
 			{
-				maki_log(serv, target, "<%s> %s", maki_user_nick(user), message);
+				maki_server_log(serv, target, "<%s> %s", maki_user_nick(user), message);
 			}
 			else
 			{
-				maki_log(serv, maki_user_nick(user), "<%s> %s", maki_user_nick(user), message);
+				maki_server_log(serv, maki_user_nick(user), "<%s> %s", maki_user_nick(user), message);
 			}
 
-			maki_dbus_emit_message(timestamp, maki_server_name(serv), maki_user_from(user), target, message);
+			maki_dbus_emit_message(maki_server_name(serv), maki_user_from(user), target, message);
 		}
 	}
 
 	g_strfreev(tmp);
 }
 
-static void maki_in_join (makiServer* serv, glong timestamp, makiUser* user, gchar* remaining)
+static void maki_in_join (makiServer* serv, makiUser* user, gchar* remaining)
 {
 	gchar* channel;
 	makiChannel* chan;
@@ -389,7 +398,6 @@ static void maki_in_join (makiServer* serv, glong timestamp, makiUser* user, gch
 	{
 		if (chan != NULL)
 		{
-			maki_channel_remove_users(chan);
 			maki_channel_set_joined(chan, TRUE);
 		}
 		else
@@ -400,22 +408,22 @@ static void maki_in_join (makiServer* serv, glong timestamp, makiUser* user, gch
 			maki_server_add_channel(serv, channel, chan);
 		}
 
-		maki_log(serv, channel, _("» You join."));
+		maki_server_log(serv, channel, _("» You join."));
 	}
 	else
 	{
-		maki_log(serv, channel, _("» %s joins."), maki_user_nick(user));
+		maki_server_log(serv, channel, _("» %s joins."), maki_user_nick(user));
 	}
 
 	if (chan != NULL)
 	{
-		maki_channel_add_user(chan, maki_user_nick(user), maki_channel_user_new(user));
+		maki_channel_add_user(chan, maki_user_nick(user));
 	}
 
-	maki_dbus_emit_join(timestamp, maki_server_name(serv), maki_user_from(user), channel);
+	maki_dbus_emit_join(maki_server_name(serv), maki_user_from(user), channel);
 }
 
-static void maki_in_part (makiServer* serv, glong timestamp, makiUser* user, gchar* remaining)
+static void maki_in_part (makiServer* serv, makiUser* user, gchar* remaining)
 {
 	gchar** tmp;
 	gchar* channel;
@@ -452,7 +460,6 @@ static void maki_in_part (makiServer* serv, glong timestamp, makiUser* user, gch
 			key = maki_channel_key(chan);
 
 			maki_channel_set_joined(chan, FALSE);
-			maki_channel_remove_users(chan);
 
 			if (!maki_channel_autojoin(chan) && key == NULL)
 			{
@@ -464,38 +471,38 @@ static void maki_in_part (makiServer* serv, glong timestamp, makiUser* user, gch
 
 		if (message != NULL)
 		{
-			maki_log(serv, channel, _("« You part (%s)."), message);
+			maki_server_log(serv, channel, _("« You part (%s)."), message);
 		}
 		else
 		{
-			maki_log(serv, channel, _("« You part."));
+			maki_server_log(serv, channel, _("« You part."));
 		}
 	}
 	else
 	{
 		if (message != NULL)
 		{
-			maki_log(serv, channel, _("« %s parts (%s)."), maki_user_nick(user), message);
+			maki_server_log(serv, channel, _("« %s parts (%s)."), maki_user_nick(user), message);
 		}
 		else
 		{
-			maki_log(serv, channel, _("« %s parts."), maki_user_nick(user));
+			maki_server_log(serv, channel, _("« %s parts."), maki_user_nick(user));
 		}
 	}
 
 	if (message != NULL)
 	{
-		maki_dbus_emit_part(timestamp, maki_server_name(serv), maki_user_from(user), channel, message);
+		maki_dbus_emit_part(maki_server_name(serv), maki_user_from(user), channel, message);
 	}
 	else
 	{
-		maki_dbus_emit_part(timestamp, maki_server_name(serv), maki_user_from(user), channel, "");
+		maki_dbus_emit_part(maki_server_name(serv), maki_user_from(user), channel, "");
 	}
 
 	g_strfreev(tmp);
 }
 
-static void maki_in_quit (makiServer* serv, glong timestamp, makiUser* user, gchar* remaining)
+static void maki_in_quit (makiServer* serv, makiUser* user, gchar* remaining)
 {
 	GHashTableIter iter;
 	gpointer key, value;
@@ -516,11 +523,11 @@ static void maki_in_quit (makiServer* serv, glong timestamp, makiUser* user, gch
 		{
 			if (remaining)
 			{
-				maki_log(serv, chan_name, _("« %s quits (%s)."), maki_user_nick(user), maki_remove_colon(remaining));
+				maki_server_log(serv, chan_name, _("« %s quits (%s)."), maki_user_nick(user), maki_remove_colon(remaining));
 			}
 			else
 			{
-				maki_log(serv, chan_name, _("« %s quits."), maki_user_nick(user));
+				maki_server_log(serv, chan_name, _("« %s quits."), maki_user_nick(user));
 			}
 		}
 
@@ -529,15 +536,15 @@ static void maki_in_quit (makiServer* serv, glong timestamp, makiUser* user, gch
 
 	if (remaining)
 	{
-		maki_dbus_emit_quit(timestamp, maki_server_name(serv), maki_user_from(user), maki_remove_colon(remaining));
+		maki_dbus_emit_quit(maki_server_name(serv), maki_user_from(user), maki_remove_colon(remaining));
 	}
 	else
 	{
-		maki_dbus_emit_quit(timestamp, maki_server_name(serv), maki_user_from(user), "");
+		maki_dbus_emit_quit(maki_server_name(serv), maki_user_from(user), "");
 	}
 }
 
-static void maki_in_kick (makiServer* serv, glong timestamp, makiUser* user, gchar* remaining)
+static void maki_in_kick (makiServer* serv, makiUser* user, gchar* remaining)
 {
 	gchar** tmp;
 	gchar* channel;
@@ -587,38 +594,38 @@ static void maki_in_kick (makiServer* serv, glong timestamp, makiUser* user, gch
 
 		if (message != NULL)
 		{
-			maki_log(serv, channel, _("« %s kicks you (%s)."), maki_user_nick(user), message);
+			maki_server_log(serv, channel, _("« %s kicks you (%s)."), maki_user_nick(user), message);
 		}
 		else
 		{
-			maki_log(serv, channel, _("« %s kicks you."), maki_user_nick(user));
+			maki_server_log(serv, channel, _("« %s kicks you."), maki_user_nick(user));
 		}
 	}
 	else
 	{
 		if (message != NULL)
 		{
-			maki_log(serv, channel, _("« %s kicks %s (%s)."), maki_user_nick(user), who, message);
+			maki_server_log(serv, channel, _("« %s kicks %s (%s)."), maki_user_nick(user), who, message);
 		}
 		else
 		{
-			maki_log(serv, channel, _("« %s kicks %s."), maki_user_nick(user), who);
+			maki_server_log(serv, channel, _("« %s kicks %s."), maki_user_nick(user), who);
 		}
 	}
 
 	if (message != NULL)
 	{
-		maki_dbus_emit_kick(timestamp, maki_server_name(serv), maki_user_from(user), channel, who, message);
+		maki_dbus_emit_kick(maki_server_name(serv), maki_user_from(user), channel, who, message);
 	}
 	else
 	{
-		maki_dbus_emit_kick(timestamp, maki_server_name(serv), maki_user_from(user), channel, who, "");
+		maki_dbus_emit_kick(maki_server_name(serv), maki_user_from(user), channel, who, "");
 	}
 
 	g_strfreev(tmp);
 }
 
-static void maki_in_nick (makiServer* serv, glong timestamp, makiUser* user, gchar* remaining)
+static void maki_in_nick (makiServer* serv, makiUser* user, gchar* remaining)
 {
 	gboolean own;
 	gchar* new_nick;
@@ -639,37 +646,34 @@ static void maki_in_nick (makiServer* serv, glong timestamp, makiUser* user, gch
 	{
 		const gchar* chan_name = key;
 		makiChannel* chan = value;
-		makiChannelUser* cuser;
 
 		if (!maki_channel_joined(chan))
 		{
 			continue;
 		}
 
-		if ((cuser = maki_channel_get_user(chan, maki_user_nick(user))) != NULL)
+		if (maki_channel_rename_user(chan, maki_user_nick(user), new_nick) != NULL)
 		{
-			maki_channel_rename_user(chan, maki_user_nick(user), new_nick);
-
 			if (own)
 			{
-				maki_log(serv, chan_name, _("• You are now known as %s."), new_nick);
+				maki_server_log(serv, chan_name, _("• You are now known as %s."), new_nick);
 			}
 			else
 			{
-				maki_log(serv, chan_name, _("• %s is now known as %s."), maki_user_nick(user), new_nick);
+				maki_server_log(serv, chan_name, _("• %s is now known as %s."), maki_user_nick(user), new_nick);
 			}
 		}
 	}
 
-	maki_dbus_emit_nick(timestamp, maki_server_name(serv), maki_user_from(user), new_nick);
-	maki_user_set_nick(user, new_nick);
+	maki_dbus_emit_nick(maki_server_name(serv), maki_user_from(user), new_nick);
+	maki_server_rename_user(serv, maki_user_nick(user), new_nick);
 
 	if (own)
 	{
 		gchar* initial_nick;
 
 		/* FIXME redundant */
-		maki_user_set_nick(maki_server_user(serv), new_nick);
+		maki_server_rename_user(serv, maki_user_nick(maki_server_user(serv)), new_nick);
 
 		initial_nick = maki_server_config_get_string(serv, "server", "nick");
 
@@ -679,12 +683,10 @@ static void maki_in_nick (makiServer* serv, glong timestamp, makiUser* user, gch
 		}
 
 		g_free(initial_nick);
-
-		own = TRUE;
 	}
 }
 
-static void maki_in_notice (makiServer* serv, glong timestamp, makiUser* user, gchar* remaining)
+static void maki_in_notice (makiServer* serv, makiUser* user, gchar* remaining)
 {
 	gchar** tmp;
 	gchar* target;
@@ -703,20 +705,20 @@ static void maki_in_notice (makiServer* serv, glong timestamp, makiUser* user, g
 	{
 		if (maki_is_channel(serv, target))
 		{
-			maki_log(serv, target, "-%s- %s", maki_user_nick(user), message);
+			maki_server_log(serv, target, "-%s- %s", maki_user_nick(user), message);
 		}
 		else
 		{
-			maki_log(serv, maki_user_nick(user), "-%s- %s", maki_user_nick(user), message);
+			maki_server_log(serv, maki_user_nick(user), "-%s- %s", maki_user_nick(user), message);
 		}
 
-		maki_dbus_emit_notice(timestamp, maki_server_name(serv), maki_user_from(user), target, message);
+		maki_dbus_emit_notice(maki_server_name(serv), maki_user_from(user), target, message);
 	}
 
 	g_strfreev(tmp);
 }
 
-static void maki_in_mode (makiServer* serv, glong timestamp, makiUser* user, gchar* remaining, gboolean is_numeric)
+static void maki_in_mode (makiServer* serv, makiUser* user, gchar* remaining, gboolean is_numeric)
 {
 	gboolean own;
 	gchar** modes;
@@ -770,40 +772,33 @@ static void maki_in_mode (makiServer* serv, glong timestamp, makiUser* user, gch
 				if ((pos = maki_prefix_position(serv, FALSE, *mode)) >= 0)
 				{
 					makiChannel* chan;
-					makiChannelUser* cuser;
+					makiUser* muser;
 
 					if ((chan = maki_server_get_channel(serv, target)) != NULL
-					    && (cuser = maki_channel_get_user(chan, modes[i])) != NULL)
+					    && (muser = maki_channel_get_user(chan, modes[i])) != NULL)
 					{
-						if (sign == '+')
-						{
-							cuser->prefix |= (1 << pos);
-						}
-						else
-						{
-							cuser->prefix &= ~(1 << pos);
-						}
+						maki_channel_set_user_prefix(chan, muser, pos, (sign == '+'));
 					}
 				}
 
 				if (is_numeric)
 				{
-					maki_log(serv, target, _("• Mode: %s %s"), buffer, modes[i]);
+					maki_server_log(serv, target, _("• Mode: %s %s"), buffer, modes[i]);
 
-					maki_dbus_emit_mode(timestamp, maki_server_name(serv), "", target, buffer, modes[i]);
+					maki_dbus_emit_mode(maki_server_name(serv), "", target, buffer, modes[i]);
 				}
 				else
 				{
 					if (own)
 					{
-						maki_log(serv, target, _("• You set mode: %s %s"), buffer, modes[i]);
+						maki_server_log(serv, target, _("• You set mode: %s %s"), buffer, modes[i]);
 					}
 					else
 					{
-						maki_log(serv, target, _("• %s sets mode: %s %s"), maki_user_nick(user), buffer, modes[i]);
+						maki_server_log(serv, target, _("• %s sets mode: %s %s"), maki_user_nick(user), buffer, modes[i]);
 					}
 
-					maki_dbus_emit_mode(timestamp, maki_server_name(serv), maki_user_from(user), target, buffer, modes[i]);
+					maki_dbus_emit_mode(maki_server_name(serv), maki_user_from(user), target, buffer, modes[i]);
 				}
 
 				++i;
@@ -812,22 +807,22 @@ static void maki_in_mode (makiServer* serv, glong timestamp, makiUser* user, gch
 			{
 				if (is_numeric)
 				{
-					maki_log(serv, target, _("• Mode: %s"), buffer);
+					maki_server_log(serv, target, _("• Mode: %s"), buffer);
 
-					maki_dbus_emit_mode(timestamp, maki_server_name(serv), "", target, buffer, "");
+					maki_dbus_emit_mode(maki_server_name(serv), "", target, buffer, "");
 				}
 				else
 				{
 					if (own)
 					{
-						maki_log(serv, target, _("• You set mode: %s"), buffer);
+						maki_server_log(serv, target, _("• You set mode: %s"), buffer);
 					}
 					else
 					{
-						maki_log(serv, target, _("• %s sets mode: %s"), maki_user_nick(user), buffer);
+						maki_server_log(serv, target, _("• %s sets mode: %s"), maki_user_nick(user), buffer);
 					}
 
-					maki_dbus_emit_mode(timestamp, maki_server_name(serv), maki_user_from(user), target, buffer, "");
+					maki_dbus_emit_mode(maki_server_name(serv), maki_user_from(user), target, buffer, "");
 				}
 			}
 		}
@@ -837,7 +832,7 @@ static void maki_in_mode (makiServer* serv, glong timestamp, makiUser* user, gch
 	g_strfreev(modes);
 }
 
-static void maki_in_invite (makiServer* serv, glong timestamp, makiUser* user, gchar* remaining, gboolean is_numeric)
+static void maki_in_invite (makiServer* serv, makiUser* user, gchar* remaining, gboolean is_numeric)
 {
 	gchar** tmp;
 	gchar* channel;
@@ -861,21 +856,21 @@ static void maki_in_invite (makiServer* serv, glong timestamp, makiUser* user, g
 
 	if (is_numeric)
 	{
-		maki_log(serv, channel, _("• You successfully invite %s."), who);
+		maki_server_log(serv, channel, _("• You successfully invite %s."), who);
 
-		maki_dbus_emit_invite(timestamp, maki_server_name(serv), "", channel, who);
+		maki_dbus_emit_invite(maki_server_name(serv), "", channel, who);
 	}
 	else
 	{
-		maki_log(serv, channel, _("• %s invites %s."), maki_user_nick(user), who);
+		maki_server_log(serv, channel, _("• %s invites %s."), maki_user_nick(user), who);
 
-		maki_dbus_emit_invite(timestamp, maki_server_name(serv), maki_user_from(user), channel, who);
+		maki_dbus_emit_invite(maki_server_name(serv), maki_user_from(user), channel, who);
 	}
 
 	g_strfreev(tmp);
 }
 
-static void maki_in_topic (makiServer* serv, glong timestamp, makiUser* user, gchar* remaining, gboolean is_numeric)
+static void maki_in_topic (makiServer* serv, makiUser* user, gchar* remaining, gboolean is_numeric)
 {
 	gchar** tmp;
 	gchar* channel;
@@ -905,28 +900,28 @@ static void maki_in_topic (makiServer* serv, glong timestamp, makiUser* user, gc
 
 	if (is_numeric)
 	{
-		maki_log(serv, channel, _("• Topic: %s"), topic);
+		maki_server_log(serv, channel, _("• Topic: %s"), topic);
 
-		maki_dbus_emit_topic(timestamp, maki_server_name(serv), "", channel, topic);
+		maki_dbus_emit_topic(maki_server_name(serv), "", channel, topic);
 	}
 	else
 	{
 		if (g_ascii_strcasecmp(maki_user_nick(user), maki_user_nick(maki_server_user(serv))) == 0)
 		{
-			maki_log(serv, channel, _("• You change the topic: %s"), topic);
+			maki_server_log(serv, channel, _("• You change the topic: %s"), topic);
 		}
 		else
 		{
-			maki_log(serv, channel, _("• %s changes the topic: %s"), maki_user_nick(user), topic);
+			maki_server_log(serv, channel, _("• %s changes the topic: %s"), maki_user_nick(user), topic);
 		}
 
-		maki_dbus_emit_topic(timestamp, maki_server_name(serv), maki_user_from(user), channel, topic);
+		maki_dbus_emit_topic(maki_server_name(serv), maki_user_from(user), channel, topic);
 	}
 
 	g_strfreev(tmp);
 }
 
-static void maki_in_rpl_namreply (makiServer* serv, glong timestamp, gchar* remaining, gboolean is_end)
+static void maki_in_rpl_namreply (makiServer* serv, gchar* remaining, gboolean is_end)
 {
 	gchar** nicks;
 	gchar** prefixes;
@@ -954,7 +949,7 @@ static void maki_in_rpl_namreply (makiServer* serv, glong timestamp, gchar* rema
 
 		nicks[0] = NULL;
 
-		maki_dbus_emit_names(timestamp, maki_server_name(serv), tmp[0], nicks, nicks);
+		maki_dbus_emit_names(maki_server_name(serv), tmp[0], nicks, nicks);
 
 		g_free(nicks);
 	}
@@ -983,7 +978,6 @@ static void maki_in_rpl_namreply (makiServer* serv, glong timestamp, gchar* rema
 				guint prefix = 0;
 				gint pos;
 				makiUser* user;
-				makiChannelUser* cuser;
 
 				prefix_str[0] = '\0';
 				prefix_str[1] = '\0';
@@ -992,26 +986,21 @@ static void maki_in_rpl_namreply (makiServer* serv, glong timestamp, gchar* rema
 				{
 					if (prefix_str[0] == '\0')
 					{
-						prefix_str[0] = serv->support.prefix.prefixes[pos];
+						prefix_str[0] = maki_server_support(serv, MAKI_SERVER_SUPPORT_PREFIX_PREFIXES)[pos];
 					}
 
 					prefix |= (1 << pos);
 					nick++;
 				}
 
-				user = maki_user_new(serv, nick);
-
-				cuser = maki_channel_user_new(user);
-				maki_channel_add_user(chan, nick, cuser);
-				cuser->prefix = prefix;
+				user = maki_channel_add_user(chan, nick);
+				maki_channel_set_user_prefix_override(chan, user, prefix);
 
 				nicks[j] = nick;
 				prefixes[j] = g_strdup(prefix_str);
-
-				maki_user_unref(user);
 			}
 
-			maki_dbus_emit_names(timestamp, maki_server_name(serv), tmp[1], nicks, prefixes);
+			maki_dbus_emit_names(maki_server_name(serv), tmp[1], nicks, prefixes);
 
 			g_strfreev(prefixes);
 			g_free(nicks);
@@ -1022,7 +1011,7 @@ static void maki_in_rpl_namreply (makiServer* serv, glong timestamp, gchar* rema
 }
 
 /* FIXME handle more stuff */
-static void maki_in_rpl_whoreply (makiServer* serv, glong timestamp, gchar* remaining, gboolean is_end)
+static void maki_in_rpl_whoreply (makiServer* serv, gchar* remaining, gboolean is_end)
 {
 	gchar** tmp;
 	guint length;
@@ -1058,7 +1047,7 @@ static void maki_in_rpl_whoreply (makiServer* serv, glong timestamp, gchar* rema
 			{
 				maki_user_set_away(user, away);
 
-				maki_dbus_emit_user_away(timestamp, maki_server_name(serv), maki_user_from(user), maki_user_away(user));
+				maki_dbus_emit_user_away(maki_server_name(serv), maki_user_from(user), maki_user_away(user));
 			}
 		}
 	}
@@ -1066,7 +1055,7 @@ static void maki_in_rpl_whoreply (makiServer* serv, glong timestamp, gchar* rema
 	g_strfreev(tmp);
 }
 
-static void maki_in_rpl_away (makiServer* serv, glong timestamp, gchar* remaining)
+static void maki_in_rpl_away (makiServer* serv, gchar* remaining)
 {
 	gchar** tmp;
 
@@ -1079,13 +1068,13 @@ static void maki_in_rpl_away (makiServer* serv, glong timestamp, gchar* remainin
 
 	if (g_strv_length(tmp) >= 2)
 	{
-		maki_dbus_emit_away_message(timestamp, maki_server_name(serv), tmp[0], maki_remove_colon(tmp[1]));
+		maki_dbus_emit_away_message(maki_server_name(serv), tmp[0], maki_remove_colon(tmp[1]));
 	}
 
 	g_strfreev(tmp);
 }
 
-static void maki_in_rpl_isupport (makiServer* serv, glong timestamp, gchar* remaining)
+static void maki_in_rpl_isupport (makiServer* serv, gchar* remaining)
 {
 	guint i;
 	guint length;
@@ -1118,13 +1107,11 @@ static void maki_in_rpl_isupport (makiServer* serv, glong timestamp, gchar* rema
 
 		if (strncmp(support[0], "CHANMODES", 9) == 0)
 		{
-			g_free(serv->support.chanmodes);
-			serv->support.chanmodes = g_strdup(support[1]);
+			maki_server_set_support(serv, MAKI_SERVER_SUPPORT_CHANMODES, support[1]);
 		}
 		else if (strncmp(support[0], "CHANTYPES", 9) == 0)
 		{
-			g_free(serv->support.chantypes);
-			serv->support.chantypes = g_strdup(support[1]);
+			maki_server_set_support(serv, MAKI_SERVER_SUPPORT_CHANTYPES, support[1]);
 		}
 		else if (strncmp(support[0], "PREFIX", 6) == 0)
 		{
@@ -1135,10 +1122,8 @@ static void maki_in_rpl_isupport (makiServer* serv, glong timestamp, gchar* rema
 			if (support[1][0] == '(' && paren != NULL)
 			{
 				*paren = '\0';
-				g_free(serv->support.prefix.modes);
-				g_free(serv->support.prefix.prefixes);
-				serv->support.prefix.modes = g_strdup(support[1] + 1);
-				serv->support.prefix.prefixes = g_strdup(paren + 1);
+				maki_server_set_support(serv, MAKI_SERVER_SUPPORT_PREFIX_MODES, support[1] + 1);
+				maki_server_set_support(serv, MAKI_SERVER_SUPPORT_PREFIX_PREFIXES, paren + 1);
 			}
 		}
 
@@ -1148,7 +1133,7 @@ static void maki_in_rpl_isupport (makiServer* serv, glong timestamp, gchar* rema
 	g_strfreev(tmp);
 }
 
-static void maki_in_rpl_motd (makiServer* serv, glong timestamp, gchar* remaining, gboolean is_end)
+static void maki_in_rpl_motd (makiServer* serv, gchar* remaining, gboolean is_end)
 {
 	if (!remaining)
 	{
@@ -1157,15 +1142,15 @@ static void maki_in_rpl_motd (makiServer* serv, glong timestamp, gchar* remainin
 
 	if (is_end)
 	{
-		maki_dbus_emit_motd(timestamp, maki_server_name(serv), "");
+		maki_dbus_emit_motd(maki_server_name(serv), "");
 	}
 	else
 	{
-		maki_dbus_emit_motd(timestamp, maki_server_name(serv), maki_remove_colon(remaining));
+		maki_dbus_emit_motd(maki_server_name(serv), maki_remove_colon(remaining));
 	}
 }
 
-static void maki_in_rpl_list (makiServer* serv, glong timestamp, gchar* remaining, gboolean is_end)
+static void maki_in_rpl_list (makiServer* serv, gchar* remaining, gboolean is_end)
 {
 	if (!remaining)
 	{
@@ -1174,7 +1159,7 @@ static void maki_in_rpl_list (makiServer* serv, glong timestamp, gchar* remainin
 
 	if (is_end)
 	{
-		maki_dbus_emit_list(timestamp, maki_server_name(serv), "", -1, "");
+		maki_dbus_emit_list(maki_server_name(serv), "", -1, "");
 	}
 	else
 	{
@@ -1184,14 +1169,14 @@ static void maki_in_rpl_list (makiServer* serv, glong timestamp, gchar* remainin
 
 		if (g_strv_length(tmp) == 3)
 		{
-			maki_dbus_emit_list(timestamp, maki_server_name(serv), tmp[0], atol(tmp[1]), maki_remove_colon(tmp[2]));
+			maki_dbus_emit_list(maki_server_name(serv), tmp[0], atol(tmp[1]), maki_remove_colon(tmp[2]));
 		}
 
 		g_strfreev(tmp);
 	}
 }
 
-static void maki_in_rpl_banlist (makiServer* serv, glong timestamp, gchar* remaining, gboolean is_end)
+static void maki_in_rpl_banlist (makiServer* serv, gchar* remaining, gboolean is_end)
 {
 	guint length;
 	gchar** tmp;
@@ -1208,26 +1193,26 @@ static void maki_in_rpl_banlist (makiServer* serv, glong timestamp, gchar* remai
 	{
 		if (length >= 1)
 		{
-			maki_dbus_emit_banlist(timestamp, maki_server_name(serv), tmp[0], "", "", -1);
+			maki_dbus_emit_banlist(maki_server_name(serv), tmp[0], "", "", -1);
 		}
 	}
 	else
 	{
 		if (length == 4)
 		{
-			maki_dbus_emit_banlist(timestamp, maki_server_name(serv), tmp[0], tmp[1], tmp[2], atol(tmp[3]));
+			maki_dbus_emit_banlist(maki_server_name(serv), tmp[0], tmp[1], tmp[2], atol(tmp[3]));
 		}
 		else if (length == 2)
 		{
 			/* This is what the RFC specifies. */
-			maki_dbus_emit_banlist(timestamp, maki_server_name(serv), tmp[0], tmp[1], "", 0);
+			maki_dbus_emit_banlist(maki_server_name(serv), tmp[0], tmp[1], "", 0);
 		}
 	}
 
 	g_strfreev(tmp);
 }
 
-static void maki_in_rpl_whois (makiServer* serv, glong timestamp, gchar* remaining, gboolean is_end)
+static void maki_in_rpl_whois (makiServer* serv, gchar* remaining, gboolean is_end)
 {
 	gchar** tmp;
 
@@ -1242,18 +1227,18 @@ static void maki_in_rpl_whois (makiServer* serv, glong timestamp, gchar* remaini
 	{
 		if (is_end)
 		{
-			maki_dbus_emit_whois(timestamp, maki_server_name(serv), tmp[0], "");
+			maki_dbus_emit_whois(maki_server_name(serv), tmp[0], "");
 		}
 		else
 		{
-			maki_dbus_emit_whois(timestamp, maki_server_name(serv), tmp[0], tmp[1]);
+			maki_dbus_emit_whois(maki_server_name(serv), tmp[0], tmp[1]);
 		}
 	}
 
 	g_strfreev(tmp);
 }
 
-static void maki_in_err_nosuch (makiServer* serv, glong timestamp, gchar* remaining, gint numeric)
+static void maki_in_err_nosuch (makiServer* serv, gchar* remaining, gint numeric)
 {
 	const gchar* reason = "";
 	const gchar* type = "";
@@ -1297,15 +1282,15 @@ static void maki_in_err_nosuch (makiServer* serv, glong timestamp, gchar* remain
 
 	arguments = i_strv_new(NULL, tmp[0], NULL);
 
-	maki_dbus_emit_error(timestamp, maki_server_name(serv), "no_such", reason, arguments);
-	maki_dbus_emit_no_such(timestamp, maki_server_name(serv), tmp[0], type);
+	maki_dbus_emit_error(maki_server_name(serv), "no_such", reason, arguments);
+	maki_dbus_emit_no_such(maki_server_name(serv), tmp[0], type);
 
 	g_free(arguments);
 
 	g_strfreev(tmp);
 }
 
-static void maki_in_err_cannot_join (makiServer* serv, glong timestamp, gchar* remaining, gint numeric)
+static void maki_in_err_cannot_join (makiServer* serv, gchar* remaining, gint numeric)
 {
 	const gchar* reason = "";
 	const gchar* type = "";
@@ -1354,15 +1339,15 @@ static void maki_in_err_cannot_join (makiServer* serv, glong timestamp, gchar* r
 
 	arguments = i_strv_new(NULL, tmp[0], NULL);
 
-	maki_dbus_emit_error(timestamp, maki_server_name(serv), "cannot_join", reason, arguments);
-	maki_dbus_emit_cannot_join(timestamp, maki_server_name(serv), tmp[0], type);
+	maki_dbus_emit_error(maki_server_name(serv), "cannot_join", reason, arguments);
+	maki_dbus_emit_cannot_join(maki_server_name(serv), tmp[0], type);
 
 	g_free(arguments);
 
 	g_strfreev(tmp);
 }
 
-static void maki_in_err_chanoprivsneeded (makiServer* serv, glong timestamp, gchar* remaining)
+static void maki_in_err_chanoprivsneeded (makiServer* serv, gchar* remaining)
 {
 	gchar** arguments;
 	gchar** tmp;
@@ -1382,7 +1367,7 @@ static void maki_in_err_chanoprivsneeded (makiServer* serv, glong timestamp, gch
 
 	arguments = i_strv_new(NULL, tmp[0], NULL);
 
-	maki_dbus_emit_error(timestamp, maki_server_name(serv), "privilege", "channel_operator", arguments);
+	maki_dbus_emit_error(maki_server_name(serv), "privilege", "channel_operator", arguments);
 
 	g_free(arguments);
 
@@ -1435,9 +1420,6 @@ void maki_in_callback (const gchar* message, gpointer data)
 		gchar* remaining;
 		guint from_length;
 		makiUser* user;
-		GTimeVal timeval;
-
-		g_get_current_time(&timeval);
 
 		parts = g_strsplit(message + 1, " ", 3);
 
@@ -1483,7 +1465,7 @@ void maki_in_callback (const gchar* message, gpointer data)
 		type = parts[1];
 		remaining = parts[2];
 
-		user = maki_user_new(serv, from[0]);
+		user = maki_server_add_user(serv, from[0]);
 
 		if (from_length > 2)
 		{
@@ -1512,46 +1494,46 @@ void maki_in_callback (const gchar* message, gpointer data)
 			{
 				/* RPL_CHANNELMODEIS */
 				case 324:
-					maki_in_mode(serv, timeval.tv_sec, user, remaining, TRUE);
+					maki_in_mode(serv, user, remaining, TRUE);
 					break;
 				/* RPL_INVITING */
 				case 341:
-					maki_in_invite(serv, timeval.tv_sec, user, remaining, TRUE);
+					maki_in_invite(serv, user, remaining, TRUE);
 					break;
 				/* RPL_NAMREPLY */
 				case 353:
 				/* RPL_ENDOFNAMES */
 				case 366:
-					maki_in_rpl_namreply(serv, timeval.tv_sec, remaining, (numeric == 366));
+					maki_in_rpl_namreply(serv, remaining, (numeric == 366));
 					break;
 				/* RPL_WHOREPLY */
 				case 352:
 				/* RPL_ENDOFWHO */
 				case 315:
-					maki_in_rpl_whoreply(serv, timeval.tv_sec, remaining, (numeric == 315));
+					maki_in_rpl_whoreply(serv, remaining, (numeric == 315));
 					break;
 				/* RPL_UNAWAY */
 				case 305:
 					maki_user_set_away(maki_server_user(serv), FALSE);
 					maki_user_set_away_message(maki_server_user(serv), NULL);
-					maki_dbus_emit_back(timeval.tv_sec, maki_server_name(serv));
-					maki_dbus_emit_user_away(timeval.tv_sec, maki_server_name(serv), maki_user_from(maki_server_user(serv)), FALSE);
+					maki_dbus_emit_back(maki_server_name(serv));
+					maki_dbus_emit_user_away(maki_server_name(serv), maki_user_from(maki_server_user(serv)), FALSE);
 					break;
 				/* RPL_NOWAWAY */
 				case 306:
 					maki_user_set_away(maki_server_user(serv), TRUE);
-					maki_dbus_emit_away(timeval.tv_sec, maki_server_name(serv));
-					maki_dbus_emit_user_away(timeval.tv_sec, maki_server_name(serv), maki_user_from(maki_server_user(serv)), TRUE);
+					maki_dbus_emit_away(maki_server_name(serv));
+					maki_dbus_emit_user_away(maki_server_name(serv), maki_user_from(maki_server_user(serv)), TRUE);
 					break;
 				/* RPL_AWAY */
 				case 301:
-					maki_in_rpl_away(serv, timeval.tv_sec, remaining);
+					maki_in_rpl_away(serv, remaining);
 					break;
 				/* RPL_ENDOFMOTD */
 				case 376:
 				/* ERR_NOMOTD */
 				case 422:
-					serv->logged_in = TRUE;
+					maki_server_set_logged_in(serv, TRUE);
 					maki_out_nickserv(serv);
 					g_timeout_add_seconds(3, maki_join, serv);
 					maki_commands(serv);
@@ -1561,19 +1543,19 @@ void maki_in_callback (const gchar* message, gpointer data)
 						maki_out_away(serv, maki_user_away_message(maki_server_user(serv)));
 					}
 
-					maki_in_rpl_motd(serv, timeval.tv_sec, remaining, TRUE);
+					maki_in_rpl_motd(serv, remaining, TRUE);
 					break;
 				/* ERR_NICKNAMEINUSE */
 				case 433:
-					if (!serv->logged_in)
+					if (!maki_server_logged_in(serv))
 					{
 						gchar* nick;
 
 						nick = g_strconcat(maki_user_nick(maki_server_user(serv)), "_", NULL);
 
-						maki_dbus_emit_nick(timeval.tv_sec, maki_server_name(serv), maki_user_nick(maki_server_user(serv)), nick);
+						maki_dbus_emit_nick(maki_server_name(serv), maki_user_nick(maki_server_user(serv)), nick);
 
-						maki_user_set_nick(maki_server_user(serv), nick);
+						maki_server_rename_user(serv, maki_user_nick(maki_server_user(serv)), nick);
 
 						g_free(nick);
 
@@ -1587,15 +1569,15 @@ void maki_in_callback (const gchar* message, gpointer data)
 				case 402:
 				/* ERR_NOSUCHCHANNEL */
 				case 403:
-					maki_in_err_nosuch(serv, timeval.tv_sec, remaining, numeric);
+					maki_in_err_nosuch(serv, remaining, numeric);
 					break;
 				/* RPL_MOTD */
 				case 372:
-					maki_in_rpl_motd(serv, timeval.tv_sec, remaining, FALSE);
+					maki_in_rpl_motd(serv, remaining, FALSE);
 					break;
 				/* RPL_TOPIC */
 				case 332:
-					maki_in_topic(serv, timeval.tv_sec, user, remaining, TRUE);
+					maki_in_topic(serv, user, remaining, TRUE);
 					break;
 				/* RPL_WHOISUSER */
 				case 311:
@@ -1609,27 +1591,27 @@ void maki_in_callback (const gchar* message, gpointer data)
 				case 318:
 				/* RPL_WHOISCHANNELS */
 				case 319:
-					maki_in_rpl_whois(serv, timeval.tv_sec, remaining, (numeric == 318));
+					maki_in_rpl_whois(serv, remaining, (numeric == 318));
 					break;
 				/* RPL_ISUPPORT */
 				case 5:
-					maki_in_rpl_isupport(serv, timeval.tv_sec, remaining);
+					maki_in_rpl_isupport(serv, remaining);
 					break;
 				/* RPL_LIST */
 				case 322:
 				/* RPL_LISTEND */
 				case 323:
-					maki_in_rpl_list(serv, timeval.tv_sec, remaining, (numeric == 323));
+					maki_in_rpl_list(serv, remaining, (numeric == 323));
 					break;
 				/* RPL_BANLIST */
 				case 367:
 				/* RPL_ENDOFBANLIST */
 				case 368:
-					maki_in_rpl_banlist(serv, timeval.tv_sec, remaining, (numeric == 368));
+					maki_in_rpl_banlist(serv, remaining, (numeric == 368));
 					break;
 				/* RPL_YOUREOPER */
 				case 381:
-					maki_dbus_emit_oper(timeval.tv_sec, maki_server_name(serv));
+					maki_dbus_emit_oper(maki_server_name(serv));
 					break;
 				/* ERR_CHANNELISFULL */
 				case 471:
@@ -1639,10 +1621,10 @@ void maki_in_callback (const gchar* message, gpointer data)
 				case 474:
 				/* ERR_BADCHANNELKEY */
 				case 475:
-					maki_in_err_cannot_join(serv, timeval.tv_sec, remaining, numeric);
+					maki_in_err_cannot_join(serv, remaining, numeric);
 					break;
 				case 482:
-					maki_in_err_chanoprivsneeded(serv, timeval.tv_sec, remaining);
+					maki_in_err_chanoprivsneeded(serv, remaining);
 					break;
 
 				default:
@@ -1654,43 +1636,43 @@ void maki_in_callback (const gchar* message, gpointer data)
 		{
 			if (strncmp(type, "PRIVMSG", 7) == 0)
 			{
-				maki_in_privmsg(serv, timeval.tv_sec, user, remaining);
+				maki_in_privmsg(serv, user, remaining);
 			}
 			else if (strncmp(type, "JOIN", 4) == 0)
 			{
-				maki_in_join(serv, timeval.tv_sec, user, remaining);
+				maki_in_join(serv, user, remaining);
 			}
 			else if (strncmp(type, "PART", 4) == 0)
 			{
-				maki_in_part(serv, timeval.tv_sec, user, remaining);
+				maki_in_part(serv, user, remaining);
 			}
 			else if (strncmp(type, "QUIT", 4) == 0)
 			{
-				maki_in_quit(serv, timeval.tv_sec, user, remaining);
+				maki_in_quit(serv, user, remaining);
 			}
 			else if (strncmp(type, "KICK", 4) == 0)
 			{
-				maki_in_kick(serv, timeval.tv_sec, user, remaining);
+				maki_in_kick(serv, user, remaining);
 			}
 			else if (strncmp(type, "NICK", 4) == 0)
 			{
-				maki_in_nick(serv, timeval.tv_sec, user, remaining);
+				maki_in_nick(serv, user, remaining);
 			}
 			else if (strncmp(type, "NOTICE", 6) == 0)
 			{
-				maki_in_notice(serv, timeval.tv_sec, user, remaining);
+				maki_in_notice(serv, user, remaining);
 			}
 			else if (strncmp(type, "MODE", 4) == 0)
 			{
-				maki_in_mode(serv, timeval.tv_sec, user, remaining, FALSE);
+				maki_in_mode(serv, user, remaining, FALSE);
 			}
 			else if (strncmp(type, "INVITE", 6) == 0)
 			{
-				maki_in_invite(serv, timeval.tv_sec, user, remaining, FALSE);
+				maki_in_invite(serv, user, remaining, FALSE);
 			}
 			else if (strncmp(type, "TOPIC", 5) == 0)
 			{
-				maki_in_topic(serv, timeval.tv_sec, user, remaining, FALSE);
+				maki_in_topic(serv, user, remaining, FALSE);
 			}
 			else
 			{
@@ -1698,7 +1680,7 @@ void maki_in_callback (const gchar* message, gpointer data)
 			}
 		}
 
-		maki_user_unref(user);
+		maki_server_remove_user(serv, maki_user_nick(user));
 
 		g_strfreev(parts);
 		g_strfreev(from);
