@@ -256,12 +256,12 @@ def cmd_topic(serverTab, channelTab, args):
 def cmd_cleartopic(serverTab, channelTab, args):
 	"""
 		Clears the topic.
-		
+
 		Usage: /cleartopic
 	"""
 	if not channelTab or not channelTab.is_channel():
 		return gui.mgmt.myPrint("No channel is active")
-		
+
 	sushi.topic(serverTab.name, channelTab.name, "")
 
 
@@ -353,12 +353,26 @@ def cmd_names(serverTab, channelTab, args):
 				print_message(_("• End of names"))
 
 			else:
+				max_nicklen = max([len(n)+len(p)
+					for (p,n) in self.tab.nickList.get_nicks_mode()])
+
 				message = ""
 				for i in xrange(len(nicks)):
-					message += "[%s<font foreground='%s'>%s</font>]" % (
-						prefixes[i],
-						color.get_nick_color(nicks[i]),
-						nicks[i])
+					nick_color = color.get_nick_color(nicks[i])
+
+					c_start = "<font foreground='%s'>" % (nick_color)
+					c_end = "</font>"
+
+					tmp = "[{prefix}{cstart}{nick}{cend}]".format(
+						prefix=prefixes[i],
+						nick=nicks[i],
+						cstart=c_start,
+						cend=c_end)
+
+					overhead = len(tmp) - len(nicks[i]) - len(prefixes[i])
+
+					message += "{0:{align}<{fill}}".format(tmp,
+									align=" ", fill=max_nicklen + overhead)
 
 					if (i+1) % self.max_col == 0:
 						print_message(message)
@@ -617,9 +631,9 @@ def cmd_query(currentServer, currentTab, args):
 
 		# fetch and write history to query (if any)
 		tab.print_last_log()
-	else:
-		# jump to query
-		tab.switch_to()
+
+	# jump to query
+	tab.switch_to()
 
 
 def cmd_clear(currentServer, currentTab, args):
@@ -681,7 +695,7 @@ def cmd_invoke_test(currentServer, currentTab, args):
 def cmd_ignore(currentServer, currentTab, args):
 	"""
 		Ignores a user pattern.
-		
+
 		Usage: /ignore <pattern>
 	"""
 	if not args:
@@ -696,7 +710,7 @@ def cmd_ignore(currentServer, currentTab, args):
 def cmd_unignore(currentServer, currentTab, args):
 	"""
 		Unignores a user pattern.
-		
+
 		Usage: /unignore <pattern>
 	"""
 	if not args:
@@ -711,13 +725,13 @@ def cmd_unignore(currentServer, currentTab, args):
 def cmd_ignores(currentServer, currentTab, args):
 	"""
 		Shows all ignored users.
-		
+
 		Usage: /ignores
 	"""
 	if not currentServer:
 		return gui.myPrint("Could not determine server")
 	ignores = sushi.ignores(currentServer.name)
-	
+
 	if not ignores:
 		gui.mgmt.myPrint(_("No users are ignored on this server."))
 	else:
