@@ -202,6 +202,14 @@ def isHighlighted (server_tab, text):
 			return True
 	return False
 
+def action_nick_color(nick):
+	""" return the nick color if color_action_nicks is activated,
+		otherwise return the default text color
+	"""
+	if config.get_bool("colors", "color_action_nicks"):
+		return color.get_nick_color(nick)
+	return color.get_color_by_key("text_action")
+
 @types (server = basestring, name = basestring)
 def _createTab (server, name):
 	""" check if tab exists, create it if not, return the tab """
@@ -296,7 +304,7 @@ def serverConnect_cb(time, server):
 					channelTab.joined=False
 				channelTab.connected=False
 
-	tab.write(time, "Connecting...", msgtype="action")
+	tab.write(time, "Connecting...", msgtype=gui.tabs.ACTION)
 
 	gui.status.set_visible("connecting", "Connecting to %s" % server)
 
@@ -316,7 +324,7 @@ def serverConnected_cb(time, server):
 	servers = [server])[1:] if tab.is_query()]:
 		query.connected = True
 
-	tab.write(time, "Connected.", msgtype="action")
+	tab.write(time, "Connected.", msgtype=gui.tabs.ACTION)
 
 def serverMOTD_cb(time, server, message, first_time = {}):
 	""" Server is sending a MOTD.
@@ -361,7 +369,7 @@ def _report_topic(time, server, channel, topic):
 	if not tab:
 		raise Exception, "%s:%s not found." % (server, channel)
 
-	tab.write(time, message, "action", no_general_output = True)
+	tab.write(time, message, gui.tabs.ACTION, no_general_output=True)
 
 def channelTopic_cb(time, server, from_str, channel, topic):
 	"""
@@ -395,7 +403,7 @@ def channelTopic_cb(time, server, from_str, channel, topic):
 		channelTab.write(time, message % {
 								"nick": nick,
 								"topic": markup.escape(topic) },
-						"action")
+						gui.tabs.ACTION)
 
 def channelBanlist_cb(time, server, channel, mask, who, when):
 	"""
@@ -405,7 +413,7 @@ def channelBanlist_cb(time, server, channel, mask, who, when):
 		tab = gui.tabs.search_tab(server, channel))
 
 	if not mask and not who and when == -1:
-		self.tab.write(time, "End of banlist.", "action")
+		self.tab.write(time, "End of banlist.", gui.tabs.ACTION)
 		code.reset_function_attrs(channelBanlist_cb)
 
 	else:
@@ -419,7 +427,7 @@ def channelBanlist_cb(time, server, channel, mask, who, when):
 				markup.escape(mask),
 				markup.escape(who),
 				markup.escape(timestring)),
-			"action")
+			gui.tabs.ACTION)
 
 """ Callbacks of maki signals """
 
@@ -452,7 +460,7 @@ def userAwayMessage_cb(timestamp, server, nick, message):
 			_(u"• %(nick)s is away (%(message)s).") % {
 				"nick": nick,
 				"message": markup.escape(message)},
-			"action")
+			gui.tabs.ACTION)
 
 		if tab and tab.name == nick:
 			tab.printed_away_message = True
@@ -465,6 +473,9 @@ def userMessage_cb(timestamp, server, from_str, channel, message):
 	nick = parse_from(from_str)[0]
 	(server_tab, channel_tab) = gui.tabs.search_tabs(server, channel)
 
+	if server_tab == None:
+		return # happens if the target server does not exist
+
 	if nick.lower() == server_tab.nick.lower():
 		ownMessage_cb(timestamp, server, channel, message)
 		return
@@ -480,14 +491,14 @@ def userMessage_cb(timestamp, server, from_str, channel, message):
 		# of text color for the main message (would
 		# override channelPrint() highlight color)
 
-		type = "highlightmessage"
+		type = gui.tabs.HIGHMESSAGE
 		messageString = message
 		gui.mgmt.set_urgent(True)
 	else:
 		# no highlight, normal message type and
 		# text color is allowed.
 
-		type = "message"
+		type = gui.tabs.MESSAGE
 		messageString = "<font foreground='%s'>%s</font>" % (
 			color.get_text_color(nick), message)
 
@@ -525,12 +536,17 @@ def userQuery_cb(timestamp, server, from_str, message):
 
 	tab = _createTab(server, nick)
 
+	if isHighlighted(tab.server, message):
+		mtype = gui.tabs.HIGHMESSAGE
+	else:
+		mtype = gui.tabs.MESSAGE
+
 	tab.write(timestamp,
 		"&lt;<font foreground='%s' weight='bold'>%s</font>&gt; %s" % (
 			color.get_nick_color(nick),
 			markup.escape(nick),
 			markup.escape(message)
-		), "message", group_string=nick)
+		), mtype, group_string=nick)
 
 	# queries are important
 	gui.mgmt.set_urgent(True)
@@ -576,7 +592,7 @@ def userMode_cb(time, server, from_str, target, mode, param):
 			_("• Modes for %(target)s: %(mode)s") % {
 				"target":target,
 				"mode":mode},
-			"action")
+			gui.tabs.ACTION)
 
 	else:
 		actor = nick
@@ -595,9 +611,9 @@ def userMode_cb(time, server, from_str, target, mode, param):
 			if not _hide_output(server_tab, "mode"):
 
 				actor = "<font foreground='%s'>%s</font>" % (
-					color.get_nick_color(actor), actor)
+					action_nick_color(actor), actor)
 				target = "<font foreground='%s'>%s</font>" % (
-					color.get_nick_color(target), target)
+					action_nick_color(target), target)
 
 				server_tab.current_write(
 					time,
@@ -606,18 +622,18 @@ def userMode_cb(time, server, from_str, target, mode, param):
 						"mode":mode,
 						"param":param,
 						"target":target},
-					"action")
+					gui.tabs.ACTION)
 		else:
 			# suitable channel/query found, print it there
 
 			n_updatePrefix(tab, param, mode)
 
-			type = "action"
+			type = gui.tabs.ACTION
 			victim = target
 			own = (target == server_tab.nick)
 
 			if (param == server_tab.nick) or own:
-				type = "highlightaction"
+				type = gui.tabs.HIGHACTION
 			elif own:
 				victim = "you"
 
@@ -863,12 +879,12 @@ def userAction_cb(time, server, from_str, channel, action):
 	action = markup.escape(action)
 
 	if isHighlighted(server_tab, action):
-		type = "highlightaction"
+		type = gui.tabs.HIGHACTION
 		actionString = action
 		gui.mgmt.set_urgent(True)
 
 	else:
-		type = "action"
+		type = gui.tabs.ACTION
 		actionString = "<font foreground='%s'>%s</font>" % (
 			color.get_text_color(nick), action)
 
@@ -929,11 +945,11 @@ def userNick_cb(time, server, from_str, newNick):
 			continue
 
 		nickString = "<font foreground='%s' weight='bold'>%s</font>" % (
-			color.get_nick_color(nick),
+			action_nick_color(nick),
 			markup.escape(nick))
 
 		newNickString = "<font foreground='%s' weight='bold'>%s</font>" % (
-			color.get_nick_color(newNick),
+			action_nick_color(newNick),
 			markup.escape(newNick))
 
 		if doPrint:
@@ -942,7 +958,7 @@ def userNick_cb(time, server, from_str, newNick):
 					"nick": nickString,
 					"newnick": newNickString
 				},
-				"action")
+				gui.tabs.ACTION)
 
 
 def userAway_cb(time, server, from_str, away):
@@ -985,14 +1001,12 @@ def userKick_cb(time, server, from_str, channel, who, reason):
 		logging.debug("userKick: channel '%s' does not exist." % (channel))
 		return
 
-	channelString = "<font foreground='%s'>%s</font>" % (
-		color.get_text_color(channel), markup.escape(channel))
+	channelString = markup.escape(channel)
 
 	nickString = "<font foreground='%s' weight='bold'>%s</font>" % (
-		color.get_nick_color(nick), markup.escape(nick))
+		action_nick_color(nick), markup.escape(nick))
 
-	reasonString = "<font foreground='%s'>%s</font>" % (
-		color.get_text_color(nick), markup.escape(reason))
+	reasonString = markup.escape(reason)
 
 	if who == server_tab.nick:
 		tab.joined = False
@@ -1005,7 +1019,7 @@ def userKick_cb(time, server, from_str, channel, who, reason):
 					"nick": nickString,
 					"reason": reasonString })
 
-			tab.write(time, message, "highlightaction")
+			tab.write(time, message, gui.tabs.HIGHACTION)
 
 	else:
 		tab.nickList.remove_nick(who)
@@ -1027,7 +1041,7 @@ def userKick_cb(time, server, from_str, channel, who, reason):
 				"nick": nickString,
 				"reason": reasonString }
 
-			tab.write(time, message, "action")
+			tab.write(time, message, gui.tabs.ACTION)
 
 def userQuit_cb(time, server, from_str, reason):
 	"""
@@ -1076,7 +1090,7 @@ def userQuit_cb(time, server, from_str, reason):
 			if not (hideServerPrint or hideChannelPrint):
 				channelTab.write(time, message % {
 									"reason": reason},
-								"action")
+								gui.tabs.ACTION)
 
 	else: # another user quit the network
 
@@ -1089,12 +1103,10 @@ def userQuit_cb(time, server, from_str, reason):
 
 		nickString = "<font foreground='%s' weight='bold'>"\
 			"%s</font>" % (
-				color.get_nick_color(nick),
+				action_nick_color(nick),
 				markup.escape(nick))
 
-		reasonString = "<font foreground='%s'>%s</font>" % (
-			color.get_text_color(nick),
-			markup.escape(reason))
+		reasonString = markup.escape(reason)
 
 		message = message % {
 			"nick": nickString,
@@ -1116,7 +1128,7 @@ def userQuit_cb(time, server, from_str, reason):
 
 				if (not (hideChannelPrint or hideServerPrint)
 				and channelTab.name.lower() == nick.lower()):
-					channelTab.write(time, message, "action")
+					channelTab.write(time, message, gui.tabs.ACTION)
 
 				# skip nickList modification for queries
 				continue
@@ -1136,7 +1148,7 @@ def userQuit_cb(time, server, from_str, reason):
 						nickList.get_operator_count())
 
 				if not (hideServerPrint or hideChannelPrint):
-					channelTab.write(time, message, "action")
+					channelTab.write(time, message, gui.tabs.ACTION)
 
 
 def userJoin_cb(timestamp, server, from_str, channel):
@@ -1183,9 +1195,6 @@ def userJoin_cb(timestamp, server, from_str, channel):
 		if doPrint:
 
 			nickString = "You"
-			channelString = "<font foreground='%s'>%s</font>" % (
-				color.get_text_color(channel), markup.escape(channel))
-
 			message = _(u"» You have joined %(channel)s.")
 
 	else: # another one joined the channel
@@ -1198,17 +1207,11 @@ def userJoin_cb(timestamp, server, from_str, channel):
 
 		if doPrint:
 			message = _(u"» %(nick)s has joined %(channel)s.")
-
 			nickString = "<font foreground='%s' weight='bold'>"\
 				"%s</font>" % (
-					color.get_nick_color(nick),
+					action_nick_color(nick),
 					markup.escape(nick))
 
-			channelString = "<font foreground='%s'>%s</font>" % (
-				color.get_text_color(channel),
-				markup.escape(channel))
-
-
 		tab.nickList.append_nick(nick)
 
 		if tab.is_active():
@@ -1218,9 +1221,9 @@ def userJoin_cb(timestamp, server, from_str, channel):
 	if doPrint:
 		message = message % {
 			"nick": nickString,
-			"channel": channelString }
+			"channel": markup.escape(channel) }
 
-		tab.write(timestamp, message, "action")
+		tab.write(timestamp, message, gui.tabs.ACTION)
 
 def userNames_cb(timestamp, server, channel, nicks, prefixes):
 	"""
@@ -1271,6 +1274,9 @@ def userPart_cb(timestamp, server, from_str, channel, reason):
 		# tab was closed
 		return
 
+	channelString = markup.escape(channel)
+	reasonString = markup.escape(reason)
+
 	if nick == stab.nick:
 		# we parted
 
@@ -1278,12 +1284,6 @@ def userPart_cb(timestamp, server, from_str, channel, reason):
 
 		if _show_output_exclusive(stab, tab, "part", own = True):
 
-			channelString = "<font foreground='%s'>%s</font>" % (
-				color.get_text_color(channel), markup.escape(channel))
-
-			reasonString = "<font foreground='%s'>%s</font>" % (
-				color.get_text_color(nick), markup.escape(reason))
-
 			if reason:
 				message = _(u"« You have left %(channel)s (%(reason)s).")
 			else:
@@ -1293,7 +1293,7 @@ def userPart_cb(timestamp, server, from_str, channel, reason):
 						message % {
 							"channel": channelString,
 							"reason": reasonString },
-						"action")
+						gui.tabs.ACTION)
 
 	else: # another user parted
 
@@ -1310,12 +1310,6 @@ def userPart_cb(timestamp, server, from_str, channel, reason):
 				"%s</font>" % (
 				color.get_nick_color(nick), markup.escape(nick))
 
-			channelString = "<font foreground='%s'>%s</font>" % (
-				color.get_text_color(channel), markup.escape(channel))
-
-			reasonString = "<font foreground='%s'>%s</font>" % (
-				color.get_text_color(nick), markup.escape(reason))
-
 			if reason:
 				message = _(u"« %(nick)s has left %(channel)s "\
 					"(%(reason)s).")
@@ -1327,7 +1321,7 @@ def userPart_cb(timestamp, server, from_str, channel, reason):
 							"nick": nickString,
 							"channel": channelString,
 							"reason": reasonString},
-						"action")
+						gui.tabs.ACTION)
 
 
 def userError_cb(time, server, domain, reason, arguments):
