@@ -246,7 +246,8 @@ def tekka_tab_remove_cb(tab):
 def tekka_channel_topic_changed_cb(tab, topic):
 	if not tab.is_active(): return
 
-	if config.get_bool("tekka","hide_topic_if_empty"):
+	if (config.get_bool("tekka","hide_topic_if_empty")
+	and config.get_bool("tekka", "show_topic_bar")):
 		if topic:
 			gui.mgmt.visibility.show_topic_bar(True)
 		else:
@@ -730,6 +731,18 @@ def inputBar_shortcut_ctrl_c(inputBar, shortcut):
 		cb.set_text(text)
 
 
+def changeTopic_shortcut(inputBar, shortcut):
+	""" The user wants to change the topic for the current tab.
+	"""
+	channel = gui.tabs.get_current_tab()
+	if not channel or not channel.is_channel():
+		return
+
+	menu = servertree_menu.ServerTreeMenu()
+	menu.get_menu(channel)
+	menu.widgets.get_object("setTopicItem").activate()
+
+
 def serverTree_query_tooltip_cb(widget, x, y, kbdmode, tooltip):
 	""" show tooltips for treeview rows.
 
@@ -831,14 +844,19 @@ def nickList_render_nicks_cb(column, renderer, model, iter):
 
 def treemodel_rows_reordered_cb(treemodel, path, iter, new_order):
 	""" new_order is not accessible, so hack arround it... """
+
+	# explicit import because what we do is bad.
+	# there should be no one writing on current_path
+	from gui.tabs.current import _set_current_path
+
 	updated = False
 	for row in treemodel:
 		if not row[0]:
 			continue
 
-		if gui.tabs._currentPath == row[0].path and not updated:
+		if gui.tabs.get_current_path() == row[0].path and not updated:
 			# update the currentPath cache
-			gui.tabs._currentPath = row.path
+			_set_current_path(row.path)
 			updated = True
 
 		# update the tab's path cache
@@ -848,8 +866,9 @@ def treemodel_rows_reordered_cb(treemodel, path, iter, new_order):
 			if not child[0]:
 				continue
 
-			if gui.tabs._currentPath == child[0].path and not updated:
-				gui.tabs._currentPath = child.path
+			if (gui.tabs.get_current_path() == child[0].path
+			and not updated):
+				_set_current_path(child.path)
 				updated = True
 
 			# update path's tab cache
@@ -1235,6 +1254,7 @@ def setupGTK():
 			"input_search": output_shortcut_ctrl_f,
 			"input_search_further": output_shortcut_ctrl_g,
 			"input_copy": inputBar_shortcut_ctrl_c,
+			"change_topic": changeTopic_shortcut,
 			"servertree_previous": serverTree_shortcut_ctrl_Page_Up,
 			"servertree_next": serverTree_shortcut_ctrl_Page_Down,
 			"servertree_close": serverTree_shortcut_ctrl_w,
